<?php

	// set default values for newer constants that might not be defined in older constants.php files
	if (!defined("MULTIPLE_SPACES")) { define($ns."MULTIPLE_SPACES", 1); }
	if (!defined("COLLAPSABLE_GROUPS")) { define($ns."COLLAPSABLE_GROUPS", 1); }
	if (!defined("CONTENT_FILE_EXTENSIONS")) { define($ns."CONTENT_FILE_EXTENSIONS", "html,htm"); }


	function require_user($user) {
	
		// make sure we have a user ID in the session; if not, redirect to the login page
	
		if ($user == "") {
			GLOBAL $login_page;
			redirect($login_page);
		}
	}


	function require_role($role, $required_role) {
	
		// make sure we have the correct in the session; if not, set feedback accordingly and redirect to the previous page
	
		if ($role < $required_role) {
			set_feedback(0, "Your current role does not allow you to perform that action.");
			redirect();
		}
	}


	function initialize($variable_name, $input_method, $sanitize_type, $maximum_size=32000) {
	
		// $variable_name = the name of the variable to initialize
		// $input_method = get, post, or session
		// $sanitize_type = none, integer, text, html, filename, or sql
		
		// requires send_email and get_URL functions
		
		// 09/10/04 - Initial development; request access only
		// 10/03/04 - Added sanitization routines
		// 10/17/04 - Fixed a bug that prevented the sanitization routines from actually running; included input value in alert message
		// 01/11/05 - Tweaked text sanitization so that it only modifes <.*>, not just < or >; improved formatting of alert message; saved integer conversion until the end of the script to avoid type mismatches
		// 12/22/05 - Changed integer checking to use a regular expression rather than the intval function so that leading zeroes don't trigger an alert message
		// 01/13/06 - Moved final integer type change to before email alert so email alert is accurate
		// 05/19/06 - Added the ability to initialize session variables, mainly for the convenience of avoiding unset variables, although this does add another level of security
		// 05/26/06 - Moved addslashes to after string comparison to avoid triggering sanitization alerts
		// 05/29/06 - Tweaked previous change to support strings or arrays
		// 08/19/06 - Fixed support for integers in sessions by forcing session variables to strings for comparison, and fixed support for arrays of integers by forcing each element of the array, not the entire array, to an integer
		// 02/05/07 - Changed webmaster email variable from a global to a constant
		// 05/11/07 - Changed array conversion to support associative arrays; added array support to integer type; skipped all but the initial value set if sanitize type is "none"
		// 05/21/08 - Moved string forcing of session variables before the original value is saved, to avoid false positives on session integers
		
		$alert_email = WEBMASTER_EMAIL;
		
		// allow developers to use mixed case for these
		$input_method = strtoupper($input_method);
		$sanitize_type = strtolower($sanitize_type);
		
		// get the value from the requested global array, or set it to "" if it doesn't exist there
		// (apparently we can't access the global variable dynamically: ${"_".strtoupper($input_method)."[\"".$variable_name."\"]"} )
		switch ($input_method) {
		
			case "GET":
		
				if (isset($_GET[$variable_name])) {
					$variable_value = $_GET[$variable_name];
				} else {
					$variable_value = "";
				}
				break;
	
			case "POST":
		
				if (isset($_POST[$variable_name])) {
					$variable_value = $_POST[$variable_name];
				} else {
					$variable_value = "";
				}
				break;

			case "SESSION":
		
				if (isset($_SESSION["editor"][$variable_name])) {
					$variable_value = $_SESSION["editor"][$variable_name];
				} else {
					$variable_value = "";
				}
				break;

		}
		
		if ($sanitize_type != "none") {


			// in order to get consistent comparisons across input methods, convert session values to strings
			// everything else should come in as a string or an array
			// we don't have to worry about supporting arrays here because sessions don't support arrays
			if ($input_method == "SESSION") {
				$variable_value = strval($variable_value);
			}

			// save the variable value before sanitizing so we can see if we ran into anything suspicious
			$variable_value_original = $variable_value;
			
			// in order to handle arrays seamlessly, we'll treat every variable as an array, then process every element
			if (is_array($variable_value)) {
				$is_really_array = 1;
			} else {
				$variable_value = Array($variable_value);
				$is_really_array = 0;
			}
			
			foreach ($variable_value as $key => $value) {
			
				// truncate the value if it's too large
				if ($variable_value[$key] != "") { // the substr function returns 0 if the string is empty, which makes the comparison too complicated
					$variable_value[$key] = substr($variable_value[$key], 0, $maximum_size);
				}
						
				// sanitize the value
				switch ($sanitize_type) {
				
					case "integer":
					
						// remove all but numeric characters to prevent SQL injection attacks
						// but check first for a negative sign to support negative integers
						$integer_sign = (substr($variable_value[$key], 0, 1) == "-") ? "-" : "" ;
						$variable_value[$key] = preg_replace("/\D/", "", $variable_value[$key]);
						$variable_value[$key] = $integer_sign.$variable_value[$key];
						break;
						
					case "text":
					
						// change < and > to HTML entities to prevent cross-site scripting attacks
						$variable_value[$key] = preg_replace("/<([^>]*)>/", "&lt;$1&gt;", $variable_value[$key]);
						break;
						
					case "html":
					
						break;
						
					case "filename":
					
						// strip special characters and directory commands to prevent command execution, directory traversing, and references to remote files
						$variable_value[$key] = preg_replace("/[^\w-\.\/\?\&\=\%]/", "", $variable_value[$key]);
						$variable_value[$key] = preg_replace("/\.\.\//", "", $variable_value[$key]);
						$variable_value[$key] = preg_replace("/:\/\//", "", $variable_value[$key]);
						break;
						
					case "sql":
					
						// remove all but alphanumeric characters to prevent SQL injection attacks
						$variable_value[$key] = preg_replace("/\W/", "", $variable_value[$key]);
						break;
						
				}
			
			}
			
			// convert variables that weren't arrays to begin with back to strings or integers
			if (!$is_really_array) {
				$variable_value = $variable_value[0];
			}
			
			// compare the new value to the original and if it's different, send an alert
			$send_notification = ($variable_value !== $variable_value_original) ? 1 : 0 ;
			
			// make sure quotes are escaped to prevent SQL injection attacks
			if ($is_really_array) {
				foreach ($variable_value as $key => $value) {
					$variable_value[$key] = addslashes(stripslashes($variable_value[$key]));
				}
			} else {
				$variable_value = addslashes(stripslashes($variable_value));
			}
	
			// change the value to an actual integer rather than a string for the integer sanitization type
			if ($sanitize_type == "integer") {
				if ($is_really_array) {
					foreach ($variable_value as $key => $value) {
						$variable_value[$key] = intval($variable_value[$key]);
					}
				} else {
					$variable_value = intval($variable_value);
				}
			}
	
			if ($send_notification) {
				$subject = "Sanitization alert";
				$message = "Some user-submitted data has failed sanitization. We've modified the data to make it safe, but please review this incident to determine whether it shows malicious intent.\n\n";
				// we could make this message easier to read by using HTML formatting (bold headings), but then any HTML we passed in the value fields would render, which we don't want
				$message .= "Script:\n".get_URL()."\n\nInput method:\n$input_method\n\nVariable name:\n$variable_name\n\nAttempted value:\n$variable_value_original\n\nModified value:\n$variable_value\n\nSanitization type:\n$sanitize_type\n\n";
				send_email($alert_email, "", $alert_email, "", $alert_email, $subject, $message, 0); 
			}
		
		}

		return $variable_value;
	
	}


	function unregister_globals() {
		// thanks to http://us.php.net/manual/en/security.globals.php
		if (ini_get("register_globals")) {
			$sources = array("_REQUEST", "_SESSION", "_SERVER", "_ENV", "_FILES");
			foreach ($sources as $source) {
				foreach ($GLOBALS[$source] as $key => $value) {
					if ((isset($GLOBALS[$key])&&($GLOBALS[$key] === $value))) {
						unset($GLOBALS[$key]);
					}
				}
			}
		}
	}
	
	
	function implode_assoc($inner_glue, $outer_glue, $array) {
		$output = "";
		if (count($array) > 0) {
			// thanks to http://www.php.net/manual/en/function.implode.php
			foreach($array as $key => $value) {
				$output[] = $key.$inner_glue.$value;
			}
			$output = implode($outer_glue, $output);
		}
		return $output;
	}
	
	
	function explode_assoc($inner_glue, $outer_glue, $string) {
		$output = Array();
		if ($string != "") {
			$array = explode($outer_glue, $string);
			foreach($array as $pair) {
				if (strpos($pair, $inner_glue) !== false) {
					list($key, $value) = explode($inner_glue, $pair);
					if ($key) {
						$output["$key"] = $value;
					}
				}
			}
		}
		return $output;
	}
	
	
	function ajax_serialize($input) {
		// convert an associative array into an encoded string for posting via an HTTP request
		$output = array();
		foreach ($input as $key=>$value) {
			$output[] = $key."=".rawurlencode($value);
		}
		$output = implode("&", $output);
		return $output;
	}
	

	function ajax_unserialize($input) {
		// convert an encoded string into an associative array for retrieving from an HTTP response
		$input = explode("&", $input);
		$output = array();
		for ($n=0; $n < count($input); $n++) {
			$pair = explode("=", $input[$n]);
			$key = $pair[0];
			$value = rawurldecode($pair[1]);
			$output[$key] = $value;
		}
		return $output;
	}
	
	
    function set_feedback($success, $text) {
        $text = ($success) ? $text : "ERROR: $text" ; // we don't need this in this site if the feedback styles have icons in them
        $style = ($success) ? "feedback_positive" : "feedback_negative" ;
        $html = "<p class=\"$style\">$text</p>\n\n";
        
        $feedback = stripslashes(initialize("feedback", "session", "html"));
		$_SESSION["editor"]["feedback"] = $feedback.$html;
    }
    
    
    function get_feedback() {
        $feedback = stripslashes(initialize("feedback", "session", "html"));
        $_SESSION["editor"]["feedback"] = "";
        return $feedback;
    }


    function check_feedback() {
        $feedback = stripslashes(initialize("feedback", "session", "html"));
        if ($feedback) {
        	if (strpos($feedback, "ERROR") === false) {
        		$result = "positive";
        	} else {
        		$result = "negative";
        	}
        } else {
        	$result = "";
        }
        return $result;
    }


    function clear_feedback() {
        $_SESSION["editor"]["feedback"] = "";
    }


	function get_value($table, $input, $output, $key, $active=0) {
	
		// returns the value of a single table cell
		// 01/26/03 - added optional active argument; if true, will only return a value if its row is active
		// 02/16/03 - adjusted to use db.php
		// 06/05/03 - adjusted to use MYSQL_ASSOC
	
		$query = "SELECT $output FROM $table WHERE $input='$key'";
		if ($active) { $query .= " AND active=1"; }
		$result = db_do_query($query);
		if ($result != 0) {
			$row = db_get_row($result, 0);
			return $row[$output];
		} else {
			return "";
		}
		
	}
	
	
	function set_value($table, $field, $value, $key_field, $key) {
	
		// sets the value of a single table cell
	
		$result = db_do_query("UPDATE $table SET $field='$value' WHERE $key_field='$key'");
		return $result;
		
	}
	
	
	function get_row_count($table) {
	
		// returns the number of rows in a table
		// we might not need this since input and key are now optional in get_value_count
	
		$result = db_do_query("SELECT ID FROM $table");
		if ($result != 0) {
			return db_num_rows($result);
		} else {
			return "";
		}
		
	}
	
	
	function get_value_count($table, $input, $key) {
	
		// returns the number of rows that match the value
	
		// 08/26/03 - made input and key optional by adding the criteria check

		$criteria = ((!$input)||(!$key)) ? "ID>0" : "$input='$key'" ;

		$result = db_do_query("SELECT ID FROM $table WHERE $criteria");
		if ($result != 0) {
			return db_num_rows($result);
		} else {
			return "";
		}
		
	}
	
	
	function get_value_total($table, $input, $output, $key) {
	
		// returns the total of all the values in a single table column, assuming they are numbers
	
		$criteria = ((!$input)||(!$key)) ? "1=1" : "$input='$key'" ;
		$result = db_do_query("SELECT sum($output) AS value_total FROM $table WHERE $criteria");
		$row = db_get_row($result, 0);
		$value_total = $row["value_total"];
		return $value_total;
		
	}
	
	
	function get_value_list($table, $input, $output, $key, $delimiter, $sort="") {
	
		// returns the value of several cells in a single table column, separated by $delimiter
		
		// 06/17/03 - reversed order of instr parameters (BTW, if using postgres you'd need to change that function to strpos)
		// 08/26/03 - made input and key optional by adding the criteria check
		// 11/22/03 - ordered results by ID, do an exact match rather than a list intersect
		// 05/20/08 - added sort argument, returned to list intersect using FIND_IN_SET function
		
	
		$delimiter = ($delimiter) ? $delimiter : ", ";
		$criteria = ((!$input)||(!$key)) ? "ID>0" : "FIND_IN_SET($input, '$key')" ;
	
		if ($sort) {
			$sort = ($sort == 1) ? "ORDER by name" : "ORDER by $sort" ;
		} else {
			$sort = "ORDER by ID";
		}
	
		$result = db_do_query("SELECT $output FROM $table WHERE $criteria $sort");
		$count = ($result != 0) ? db_num_rows($result) : 0 ;
	
		$value_list = "";
		for ($i=0; $i < $count; $i++) {
			$row = db_get_row($result, $i);
			$this_value = $row[$output];
			$value_list .= $this_value.$delimiter;
		}
		$value_list = substr($value_list, 0, -(strlen($delimiter)));
		
		return $value_list;
		
	}
	
	
	function make_date_menus($label, $form, $field, $selected) {
	
		// creates a set of popup menus, which makes it easier to capture SQL-standard dates
		// this function requires date_month, date_day, and date_year tables in your database, which you can quickly add by importing the contents of date_tables.sql
		// it also requires the select_date() JavaScript function, from shared.js
		// you can also add the date collection routine from tidbits.php to your submit script
	
		// $label - a small text label that appears above the menus
		// $form - the name of the HTML form that these menus appear on
		// $field - the name of the database field that this date will be submitted into
		// $selected - an SQL-standard date to set the menus to initially (optional)
	
		// 02/16/03 - doesn't try to set if the default date is $no_date (useful in postgresql); we now return a value containing the menu, rather than printing the menu directly
		// 06/23/03 - renamed selectDate to select_date
		// 06/25/03 - renamed default_date to selected and added a conditional to set blank selected values
		// 09/09/03 - replaced font tags with body style
		// 09/11/03 - set a minimum and maximum $no_date, for sorting in two directions
		// 03/02/04 - added a Clear button to the top of the menus
		// 11/14/07 - added quotes around HTML attributes
		// 10/14/08 - reworked to use make_menu_simple instead of make_menu, so that we no longer need the date tables in the database
		// 11/19/08 - if there is no label, don't show the shortcut buttons, either
		
		$output = "";

		GLOBAL $no_date_min, $no_date_max, $site_start_year, $month_names;
		if (!isset($no_date_min)) { $no_date_min = "0000-00-00"; }
		if (!isset($no_date_max)) { $no_date_max = "9999-12-31"; }
		if (!isset($site_start_year)) { $site_start_year = "2008"; }
		if (!isset($month_names)) { $month_names = Array(1=>"Jan", 2=>"Feb", 3=>"Mar", 4=>"Apr", 5=>"May", 6=>"Jun", 7=>"Jul", 8=>"Aug", 9=>"Sep", 10=>"Oct", 11=>"Nov", 12=>"Dec"); }
		
		if (($selected == $no_date_min)||($selected == $no_date_max)) { $selected = ""; }
		
		if ($selected) {
			$temp = explode("-", $selected);
			$selected_year = $temp[0];
			$selected_month = $temp[1];
			$selected_day = $temp[2];
		} else {
			$selected_year = "";
			$selected_month = "";
			$selected_day = "";
		}
	
		$output .= "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
		if ($label) {
		$output .= "	<tr valign=\"bottom\">\n";
		$output .= "		<td class=\"body\">$label</td>\n";
		$output .= "		<td class=\"body\" align=\"right\">\n";
		$output .= "			<a href=\"JavaScript:select_date('$form.$field','')\">Today</a>&nbsp;\n";
		$output .= "			<a href=\"JavaScript:select_date('$form.$field','-1')\">Clear</a>\n";
		$output .= "		</td>\n";
		//$output .= "	</tr><tr valign=\"top\"><td colspan=\"2\"><img src=\"./graphics/spacer.gif\" width=\"1\" height=\"2\"></td>\n"; // turn this on for space between the label and the menus
		$output .= "	</tr>\n";
		}
		$output .= "	<tr valign=\"top\"><td colspan=\"2\">\n";
					$month_menu = make_menu_simple("Month...", $field."_month", 1, 12, 1, 0, $selected_month);
					//$month_menu = preg_replace("/>(\d+)</", ">".$month_names[$1]."<", $month_menu); // change month numbers to month names -- too bad this doesn't work!
					foreach ($month_names as $month=>$name) {
						$month_menu = str_replace(">$month<", ">$name<", $month_menu); // change month numbers to month names
					}
					$output .= $month_menu;
					$output .= "&nbsp;";
					$output .= make_menu_simple("Day...", $field."_day", 1, 31, 1, 0, $selected_day);
					$output .= "&nbsp;";
					$output .= make_menu_simple("Year...", $field."_year", $site_start_year, ($site_start_year + 6), 1, 0, $selected_year);
		$output .= "	</td></tr>\n";
		$output .= "</table>\n";
		
		return $output;
	
	}
	
	
	function combine_date_elements($year, $month, $day) {
	
		// helps us collect the values submitted by the date menus for insertion into a single database field
		// if any of the individual menus are unset, we'll put the empty date value into the database, since we can't store a date like 2004--1-15 in a standard date field
	
		if (($year != "-1")&&($month != "-1")&&($day != "-1")) {
			$date = "$year-$month-$day";
		} else {
			$date = "0000-00-00";
		}
		
		return $date;
	
	}
	
	
	function make_time_menus($label, $form, $field, $increment, $selected) {
	
		// creates a set of popup menus, which makes it easier to capture SQL-standard times
		// this function requires the select_time() JavaScript function, from shared.js
		// you can also add the time collection routine from tidbits.php to your submit script
	
		// $label - a small text label that appears above the menus
		// $form - the name of the HTML form that these menus appear on
		// $field - the name of the database field that this time will be submitted into
		// $increment - increment to be used between minutes
		// $selected - an SQL-standard time to set the menus to initially (optional)
	
		// 06/24/03 - initial development
		// 06/25/03 - renamed default_time to selected and added a conditional to set blank selected values
		// 09/09/03 - replaced font tags with body style
		// 07/26/04 - added increment
		// 09/02/04 - added class=form to the am/pm menu
		// 11/14/07 - added quotes around HTML attributes
		// 12/07/07 - removed "if $selected = '00:00:00'" so we could load midnight selections (but now we have no way to show as unset)
		// 06/04/08 - added missing $increment value to function declaration
		// 11/19/08 - if there is no label, don't show the shortcut buttons, either; added Clear button to match make_date_menus
		
		$output = "";

		$use_seconds = 0;
		
		if (!$increment) { $increment == 1; }
		
		if ($selected) {
			$temp = explode(":", $selected);
			$selected_hours = $temp[0];
			$selected_minutes = $temp[1];
			if ($use_seconds) {
				$selected_seconds = $temp[2];
			}
			if ($selected_hours >= 12) {
				$selected_ampm = "pm";
				if ($selected_hours > 12) {
					$selected_hours = $selected_hours - 12;
				}
			} else {
				$selected_ampm = "am";
				if ($selected_hours == 0) {
					$selected_hours = 12;
				}
			}
		} else {
			$selected_hours = "";
			$selected_minutes = "";
			$selected_seconds = "";
			$selected_ampm = "";
		}
		
		$output .= "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
		if ($label) {
		$output .= "	<tr valign=\"bottom\">\n";
		$output .= "		<td class=\"body\">$label</td>\n";
		$output .= "		<td class=\"body\" align=\"right\">\n";
		$output .= "			<a href=\"JavaScript:select_time('$form.$field','',$increment)\">Now</a>\n";
		$output .= "			<a href=\"JavaScript:select_time('$form.$field','-1',$increment)\">Clear</a>\n";
		$output .= "		</td>\n";
		//$output .= "	</tr><tr valign=\"top\"><td colspan=\"2\"><img src=\"../graphics/spacer.gif\" width=\"1\" height=\"2\"></td>\n"; // turn this on for space between the label and the menus
		$output .= "	</tr>\n";
		}
		$output .= "	<tr valign=\"top\"><td colspan=\"2\">\n";
					$output .= make_menu_simple("Hours...", $field."_hours", 1, 12, 1, 2, $selected_hours);
					$output .= " : ";
					$output .= make_menu_simple("Minutes...", $field."_minutes", 0, 59, $increment, 2, $selected_minutes);
					if ($use_seconds) {
						$output .= " : ";
						$output .= make_menu_simple("Seconds...", $field."_seconds", 0, 59, 1, 2, $selected_seconds);
					} else {
						$output .= "<input type=\"hidden\" name=\"".$field."_seconds\" value=\"00\">\n";
					}
					$output .= "&nbsp;";
					$am_selected = ($selected_ampm == "am") ? "selected " : "" ;
					$pm_selected  = ($selected_ampm == "pm") ? "selected " : "" ;
					$output .= "<select name=\"".$field."_ampm\" class=\"form\">\n";
					$output .= "<option value=\"-1\">am/pm...</option>\n";
					$output .= "<option value=\"am\" $am_selected >am</option>\n";
					$output .= "<option value=\"pm\" $pm_selected >pm</option>\n";
					$output .= "</select>\n";
		$output .= "	</td></tr>\n";
		$output .= "</table>\n";
		
		return $output;
	
	}
	
	
	function combine_time_elements($hours, $minutes, $seconds, $ampm) {
	
		// helps us collect the values submitted by the time menus for insertion into a single database field
		// if any of the individual menus are unset, we'll put the empty date value into the database, since we can't store a time like 10:-1:30 in a standard time field
	
		if (($hours != "-1")&&($minutes != "-1")&&($seconds != "-1")&&($ampm != "-1")) {
			if (($ampm == "pm")&&($hours < 12)) { $hours = $hours + 12; }
			if (($ampm == "am")&&($hours == 12)) { $hours = 0; }
			$time = "$hours:$minutes:$seconds";
		} else {
			$time = "00:00:00";
		}
		
		return $time;
	
	}
	
	
	function make_menu($title, $table, $name, $limit, $value, $sort, $multiple, $selected) { 
	
		// creates an HTML popup menu (<form><select>) based on the contents of a table

		// $title - the value to set for the first option, which shows by default in the menu; use "Tablename..." in most cases (recommended)
		// $table - the MySQL table to select from
		// $name - the name to set for the HTML select tag; use 0 to use the table name (default)
		// $limit - show only the rows listed; use 0 or "all" to show all active rows (default)
		// $value - set the value of each option to something other than the row ID; use 0 to set values to IDs (default)
		// $sort - if 1, sorts alphabetically by name; if column name, sorts by that column; use 0 to sort by ID (default)
		// $multiple - if 1, allows multiple selections via a text box; use 0 for a standard popup (default)
		// $selected - display the menu with this option selected by default (refers to position in the database table, not position in the menu); use 0 to show unselected (default) or "all" to show all items (when $multiple == 1)
	
		// 04/12/01 - allows inactive but selected values to appear; also limit=0 does no filtering (show all items), limit='' does total filtering (show no items)
		// 03/28/02 - added $multiple option; submits $table[] if used, $table if not
		// 05/08/02 - fixed two little bugs introduced in the last version
		// 01/26/03 - removed "WHERE active=1" from the query, so we can display inactive but selected or filtered records
		// 02/16/03 - we now return a value containing the menu, rather than printing the menu directly
		// 06/18/03 - changed the for loop to go from 0 to <$count
		// 07/24/03 - added the $select="all" option for multiple selection lists
		// 08/12/04 - added "WHERE active=1 OR active=0" for compatibility with the account_filter function in our web apps
		// 05/26/05 - added strict comparison operators to make the $selected and $limit options work with either 0 or "0"
		// 09/22/05 - added double quotes around HTML attributes
		// 07/19/07 - moved the limit functionality into the query, so we can no longer show an inactive item by including it in the $limit value; we can still show an inactive item by including it in the $selected value

		$output = "";
	
		$name = ($name) ? $name : $table ;
		$value = ($value) ? $value : "ID" ;
		
		$limit = (is_array($limit)) ? implode(",", $limit) : $limit ;
		$limit = (($limit !== 0)&&($limit != "all")) ? "WHERE FIND_IN_SET(ID, '$limit')" : "" ; // the !== here helps distinguish between 0 (no limit set) and "" (none allowed)
		
		$selected = (is_array($selected)) ? implode(",", $selected) : $selected ;
		
		if ($sort) {
			$sort = ($sort == 1) ? "ORDER by name" : "ORDER by $sort" ;
		} else {
			$sort = "ORDER by ID";
		}
	
		if ($multiple) {
			$multiple = "multiple";
			$name = $name."[]";
			$output .= "$title<br>\n";
		} else {
			$multiple = "";
		}
	
		$result = db_do_query("SELECT ID, $value, name, active FROM $table $limit $sort");
		$count = db_num_rows($result);
		
		$output .= "<select name=\"$name\" $multiple class=\"form\">\n";
		if (!$multiple) { $output .= "	 <option value=\"-1\">$title</option>\n"; } // you can set value=0 here to display nothing in views instead of the NA value
		for ($i=0; $i<$count; $i++) {
			$row = db_get_row($result, $i);
			$this_selected = ((strstr(",".$selected.",", ",".$row[$value].","))||(($selected === "all")&&($multiple))) ? " selected" : "" ; // this handles $select values that are single items or lists (when $multiple=1)
			if (($row["active"])||($this_selected)) {
				$output .= "  <option value=\"".$row[$value]."\" ".$this_selected.">".$row["name"]."</option>\n";
			}
		}
		$output .= "</select>\n\n";
		
		return $output;
	
	}
	
	
	function make_menu_simple($title, $name, $start, $end, $increment, $digits, $selected) { 
	
		// creates an HTML popup menu (<form><select>) with numeric values from $start to $end

		// $title - the value to set for the first option, which shows by default in the menu
		// $name - the name to set for the HTML select tag
		// $start - the number to use as the first value
		// $end - the number to use as the last value
		// $increment - number at which to increment the values; use 1 to increment by whole numbers (default)
		// $digits - adds leading zeros to numbers less than this length; use 0 to leave values untouched (default)
		// $selected - display the menu with this option selected by default; use 0 to show unselected (default)
	
		// 09/24/02 - initial development
		// 09/26/02 - we now return a value containing the menu, rather than printing the menu directly
		// 11/16/02 - added digits argument to add leading zeros
		// 02/16/03 - we now return a value containing the menu, rather than printing the menu directly
		// 06/25/03 - added another condition to ensure that nothing would be selected if $selected is ""
		// 02/15/04 - write options into an array and sort backwards if $start is greater than $end
		// 07/26/04 - added increment - NOTE: this adds an argument, so all function calls must be updated
		// 09/22/05 - added double quotes around HTML attributes

		$output = "";
		
		$increment = (!$increment) ? 1 : $increment;
		$selected = (is_array($selected)) ? implode(",", $selected) : $selected ;
		
		// it would have been much neater to just dynamically write the for statement below, but we couldn't figure out a way to do that
		if ($start > $end) {
			$start_temp = $start;
			$end_temp = $end;
			$start = $end_temp;
			$end = $start_temp;
			$reverse = 1;
		} else {
			$reverse = 0;
		}

		$output .= "<select name=\"$name\" class=\"form\">\n";
		$output .= "	 <option value=\"-1\">$title</option>\n"; // you can set value=0 here to display nothing in views instead of the NA value
		$options = Array();
		for ($i=$start; $i <= $end; $i++) {
			if ($i % $increment == 0){
				$this_number = strval($i);
				while (strlen($this_number) < $digits) { $this_number = "0".$this_number; }
				$this_selected = ((($i == $selected)||($this_number == $selected))&&($selected != "")) ? " selected" : "" ; // this handles $select values that are single items or lists (when $multiple=1)
				array_push($options, "  <option value=\"".$this_number."\" ".$this_selected.">".$this_number."</option>\n");
			}
		}
		if ($reverse) { $options = array_reverse($options); }
		$output .= implode("", $options);
		$output .= "</select>\n\n";
		
		return $output;
	
	}
	
	
    function make_checkboxes($table, $name, $limit, $value, $sort, $delimiter, $selected) { 
	
		// creates a series of HTML checkboxes (<form><input type=checkbox>) based on the contents of a table
	
		// $table - the MySQL table to select from
		// $name - the name to set for the HTML input tag; use 0 to use the table name (default)
		// $limit - show only the rows listed; use 0 or "all" to show all active rows (default)
		// $value - set the value of each checkbox to something other than the row ID; use 0 to set values to IDs (default)
		// $sort - if 1, sorts alphabetically by name; if column name, sorts by that column; use 0 to sort by ID (default)
		// $delimiter - what string to put in between the checkboxes; use 0 to include line breaks (default)
		// $selected - display the checkboxes with these options selected by default (refers to position in the database table, not position in the menu); use 0 to show all unselected (default)
	
		// 02/16/03 - added delimiter argument; we now return a value containing the menu, rather than printing the menu directly
		// 09/09/03 - removed class=form from the input tag since we removed it from make_radio_buttons
		// 05/24/05 - fixed $limit so it filters by ID rather than by table sequence
		// 05/26/05 - added strict comparison operators to make the $selected and $limit options work with either 0 or "0"
		// 09/22/05 - added double quotes around HTML attributes
		// 07/19/07 - moved the limit functionality into the query for better performance; moved the active check OUT of the query to allow displaying inactive but selected items; collapsed the this_checked setting to a single line; updated singleton tags for HTML 4.0 compliance
		
		$output = "";
		
		$name = ($name) ? $name : $table ;
		$value = ($value) ? $value : "ID" ;
		$delimiter = ($delimiter) ? $delimiter : "<br />\n" ;
	
		$limit = (is_array($limit)) ? implode(",", $limit) : $limit ;
		$limit = (($limit !== 0)&&($limit != "all")) ? "WHERE FIND_IN_SET(ID, '$limit')" : "" ; // the !== here helps distinguish between 0 (no limit set) and "" (none allowed)
		
		$selected = (is_array($selected)) ? implode(",", $selected) : $selected ;
	
		if ($sort) {
			$sort = ($sort == 1) ? "ORDER by name" : "ORDER by $sort" ;
		} else {
			$sort = "ORDER by ID";
		}
	
		$result = db_do_query("SELECT $value, name, active FROM $table $limit $sort");
		$count = db_num_rows($result);
		
		for ($i=0; $i<$count; $i++) {
			$row = db_get_row($result, $i);
			$this_checked = (strstr(",$selected,", ",$row[$value],")||($selected === "all")) ? "checked" : "" ;
			if (($row["active"])||($this_checked)) {
				$output .= "<input type=\"checkbox\" name=\"".$name."[]\" value=\"".$row[$value]."\" ".$this_checked." /> ".$row["name"].$delimiter."\n";
			}
		}
		
		return $output;
	
	}
    
    
    function make_radio_buttons($table, $name, $limit, $value, $sort, $delimiter, $selected) { 
	
		// creates a series of HTML radio buttons (<form><input type=radio>) based on the contents of a table
	
		// $name - the name to set for the HTML input tag; use 0 to use the table name (default)
		// $table - the MySQL table to select from
		// $limit - show only the rows listed; use 0 or "all" to show all active rows (default)
		// $value - set the value of each radio button to something other than the row ID; use 0 to set values to IDs (default)
		// $sort - if 1, sorts alphabetically by name; if column name, sorts by that column; use 0 to sort by ID (default)
		// $delimiter - what string to put in between the radio buttons; use 0 to include line breaks (default)
		// $selected - display the buttons with this options selected by default (refers to position in the database table, not position in the menu)
	
		// 02/16/03 - added delimiter argument; we now return a value containing the menu, rather than printing the menu directly
		// 09/09/03 - removed class=form from the input tag; that created weird alignments on Windows
		// 03/10/05 - removed the square brackets from the element names, since radio buttons don't need to send arrays
		// 05/24/05 - fixed $limit so it filters by ID rather than by table sequence
		// 05/26/05 - added strict comparison operators to make the $selected and $limit options work with either 0 or "0"
		// 08/17/05 - changed default sorting to ID, not ID DESC; restored delimiter argument -- what happened to that?
		// 09/22/05 - added double quotes around HTML attributes
		// 07/19/07 - added missing $delimiter argument to argument list
		// 07/19/07 - moved the limit functionality into the query for better performance; moved the active check OUT of the query to allow displaying inactive but selected items; collapsed the this_checked setting to a single line; updated singleton tags for HTML 4.0 compliance
		
		$output = "";

		$name = ($name) ? $name : $table ;
		$value = ($value) ? $value : "ID" ;
		$delimiter = ($delimiter) ? $delimiter : "<br />\n" ;
	
		$limit = (is_array($limit)) ? implode(",", $limit) : $limit ;
		$limit = (($limit !== 0)&&($limit != "all")) ? "WHERE FIND_IN_SET(ID, '$limit')" : "" ; // the !== here helps distinguish between 0 (no limit set) and "" (none allowed)
	
		if ($sort) {
			$sort = ($sort == 1) ? "ORDER by name" : "ORDER by $sort" ;
		} else {
			$sort = "ORDER by ID";
		}
	
		$result = db_do_query("SELECT $value, name, active FROM $table $limit $sort");
		$count = db_num_rows($result);
		
		for ($i=0; $i<$count; $i++) {
			$row = db_get_row($result, $i);
			$this_checked = ($row[$value]."" === $selected."") ? "checked" : "" ; // the ."" here forces both to strings for an accurate comparison
			if (($row["active"])||($this_checked)) {
				$output .= "<input type=\"radio\" name=\"".$name."\" value=\"".$row[$value]."\" ".$this_checked." />".$row["name"].$delimiter."\n";
			}
		}
		
		return $output;
	
	}
	
	
	function prepare_filters($user_variable, $page, $names) {
	
		// gets the requested filter values from the database and updates them if they have changed
		// this assumes we have database tables called user_filter and user_sort, each with ID, user_variable, user, page, name, and value fields
		
		// $user_variable - the name of the session variable that stores the user ID; usually this is "user," but we might separate that into "administrator," "member," etc.
		// $page - the root of the page name calling this function
		// $names - a comma-delimited list of filter names
		
		$user = initialize($user_variable, "session", "integer");
		$names = explode(",", $names);

		GLOBAL $filters_submitted;
//print "filters_submitted: $filters_submitted<br>";
		foreach ($names as $name) {
			GLOBAL ${"filter_".$name};
//print "setting global: $"."filter_".$name." (".${"filter_".$name}.")<br>";
		}
		GLOBAL $sort;
	
		// get the values that are currently saved in the database
		
		$query = "SELECT name, value FROM user_filter WHERE user_variable='$user_variable' AND user='$user' AND page='$page'";
		$result = db_do_query($query);
		$count = db_num_rows($result);
		for ($i=0; $i < $count; $i++) {
			$row = db_get_row($result, $i);
			extract($row);
			${"filter_".$name."_saved"} = $value;
		}

		$query = "SELECT value FROM user_sort WHERE user_variable='$user_variable' AND user='$user' AND page='$page'";
		$result = db_do_query($query);
		$row = db_get_row($result, 0);
		$sort_saved = $row["value"];
		
		// update the saved values if we've input something different

		foreach ($names as $name) {
			${"filter_".$name."_saved"} = (isset(${"filter_".$name."_saved"})) ? ${"filter_".$name."_saved"} : "" ;
//print "TEST: $name is ".${"filter_".$name}.", $name saved is ".${"filter_".$name."_saved"}."<br>";
			if (($filters_submitted)&&(${"filter_".$name} != ${"filter_".$name."_saved"})) {
//print "TEST: updating $name<br>";
				$query = "DELETE FROM user_filter WHERE user_variable='$user_variable' AND user='$user' AND page='$page' AND name='$name'";
				db_do_query($query);
				$query = "INSERT INTO user_filter (user_variable, user, page, name, value) VALUES ('$user_variable', '$user', '$page', '$name', '".${"filter_".$name}."')";
				db_do_query($query);
			} else {
//print "TEST: reading $name<br>";
				${"filter_".$name} = ${"filter_".$name."_saved"};
			}
		}

		if (($filters_submitted)&&($sort != $sort_saved)) {
			$query = "DELETE FROM user_sort WHERE user_variable='$user_variable' AND user='$user' AND page='$page'";
			db_do_query($query);
			$query = "INSERT INTO user_sort (user_variable, user, page, value) VALUES ('$user_variable', '$user', '$page', '$sort')";
			db_do_query($query);
		} else {
			$sort = $sort_saved;
		}

	}
	

	function send_email($to_email, $to_name, $from_email, $from_name, $bounce_email, $subject, $message, $template, $custom_headers="") {
	
		// sends plain text mail, HTML-formatted mail, or HTML mail merged with an HTML template
	
		if ($template == "0") {  // plain text
			
			$headers = "Content-Type: text/plain; charset=us-ascii";
			
		} else if ($template == "1") {  // HTML-formatted
				
			$headers = "Content-Type: text/html; charset=iso-8859-1";
			$message = str_replace("\t", "", $message);  // remove tabs, which look funny in some email programs
		
		} else {  // HTML-formatted with a MIME template
		
			// $template should be a PHP file that defines additional MIME headers and the MIME parts,
			// with "$subject" and "$message" inserted where you want the subject and message text to appear
		
			include($template);
			//$message = chunk_split($MIME_message, 72, "\r\n"); // hmm, why doesn't this work?
			//$message = wordwrap($MIME_message, 72, "\n"); // don't we need this anymore?
	
			$headers = $MIME_headers;
			$message = $MIME_message;
					
		}
		
		$to = (($to_name)&&($to_name != " ")) ? "\"$to_name\" <$to_email>" : "$to_email" ;
		$from = (($from_name)&&($from_name != " ")) ? "\"$from_name\" <$from_email>" : "$from_email" ;
	
		$result = mail($to, $subject, $message, "From: $from\nErrors-To: $bounce_email\Return-Path: $bounce_email\n".$headers.$custom_headers, "-f $bounce_email");
		
		return $result;
		
	}
	
	
	function make_pagination_links($start, $results_per_page, $total) {
	
		// adds a "start" argument to the links to show a subset of search results
		// returns an empty string if there are fewer results than the amount allowed per page
		// you can use LIMIT $start, $results_per_page to limit the query
		// you can use SQL_CALC_FOUND_ROWS to get the total, regardless of the limit, from the query
	
		// 05/21/08 - initial development
		// 08/11/08 - added pagination_previous, pagination_next and pagination_numbers classes; this lets us control spacing and set images or hide the previous/next text from the style sheets
		// 08/25/08 - updated to support links with either ? or &
		// 09/10/08 - added $max_page_links to limit the number of page links displayed at one time
		// 09/12/08 - added rounding to avoid fractional page numbers
		// 11/20/08 - changed links to call a JavaScript function rather than do direct navigation
		
		$max_page_links = 15;
		
		if ($total > $results_per_page) {
			$pagination_links = Array();
			if ($total > $results_per_page) {
				$previous = $start - $results_per_page;
				$next = $start + $results_per_page;
				$this_page = $start / $results_per_page;
				$pages = ceil($total / $results_per_page);
				if ($pages > $max_page_links) {
					$first_page = max(0, ($this_page - floor(($max_page_links / 2))));
					$display_pages = $max_page_links;
				} else {
					$first_page = 0;
					$display_pages = $pages;
				}
				if ($this_page > 0) {
					$pagination_links[] = "<a href=\"JavaScript:page('$previous');\" class=\"pagination_previous\">Previous</a>";
				} else {
					$pagination_links[] = "<span class=\"pagination_previous\">Previous</span>";
				}
				for ($i=$first_page; $i<($first_page + $display_pages); $i++) {
					$display_number = $i + 1; // everything in this section is 0-indexed, except the page numbers we show the users
					$this_start = $i * $results_per_page;
					if ($i != $this_page) {
						$pagination_links[] = "<a href=\"JavaScript:page('$this_start');\" class=\"pagination_numbers\">$display_number</a>";
					} else {
						$pagination_links[] = "<span class=\"pagination_numbers\" id=\"active\">$display_number</span>";
					}
				}
				if ($this_page < ($pages - 1)) {
					$pagination_links[] = "<a href=\"JavaScript:page('$next');\" class=\"pagination_next\">Next</a>";
				} else {
					$pagination_links[] = "<span class=\"pagination_next\">Next</span>";
				}
			}
			$pagination_links = implode("&nbsp;\n", $pagination_links);
		} else {
			$pagination_links = "";
		}
		return $pagination_links;

	}
	
	
	function find_desired_set($result, $count) {
	
		// moves forward in a query to find the result set that contains a particular record
		// requires that the result contains an ID field

		GLOBAL $start, $show, $desired_set;
		
		if ($show) {
			for ($i=0; $i < $count; $i++) {
				$row = db_get_row($result, $i);
				if ($row["ID"] == $show) {
					$desired_set = 1;
					break;
				}
			}
			if (!$desired_set) {
				if ($count) {
					$start = $start + ADMIN_RECORDS_PER_PAGE;
				} else {
					// we must have gone all the way through without finding the record, so just forget about it
					$start = 0;
					$show = 0;
				}
			}
		} else {
			$desired_set = 1;
		}
	
	}
	
	
	function build_excerpt($ID, $text, $max_length) {
	
		// instead of showing the entire text, show a portion of it with a "more" link
	
		if (strlen($text) > $max_length) {
			$text_more = $text;
			$text_less = substr($text, 0, $max_length)."...";
			$link_more = "(<a href=\"JavaScript:document.getElementById('excerpt_less_$ID').style.display = 'none'; document.getElementById('excerpt_more_$ID').style.display = 'inline';\">more</a>)";
			$link_less = "(<a href=\"JavaScript:document.getElementById('excerpt_more_$ID').style.display = 'none'; document.getElementById('excerpt_less_$ID').style.display = 'inline';\">less</a>)";
			$text = "<span style=\"display: none;\" id=\"excerpt_more_$ID\">$text_more $link_less</span>";
			$text .= "<span style=\"display: inline;\" id=\"excerpt_less_$ID\">$text_less $link_more</span>";
		}

		return $text;

	}


	function convert_characters($text, $style="html") {

		// convert special characters to HTML entities or low-ASCII equivalents
		
		// these aren't high-ASCII, but they're not allowed in XML so let's convert them first
		//if ($style == "html") {
		//	$text = str_replace("&", "&amp;", $text); // ampersand
		//	$text = str_replace("<", "&lt;", $text); // greater-than sign
		//	$text = str_replace(">", "&gt;", $text); // less-than sign
		//}
		
		// now, change all high-ASCII characters to numeric definitions
		$text_temp = "";
		for ($i=0; $i < strlen($text); $i++) {
			$decimal = ord($text[$i]);
			if ($decimal > 127) {
				$text_temp .= "&#".$decimal.";";
			} else {
				$text_temp .= $text[$i];
			}
		}
		$text = $text_temp;
		
		// finally, convert known definitions to HTML or text
		if ($style == "html") {
			$text = str_replace("&#133;", "&hellip;", $text); // ellipses
			$text = str_replace("&#145;", "&lsquo;", $text); // left single quote
			$text = str_replace("&#146;", "&rsquo;", $text); // right single quote
			$text = str_replace("&#147;", "&ldquo;", $text); // left double quote
			$text = str_replace("&#148;", "&rdquo;", $text); // right double quote
			$text = str_replace("&#149;", "&bull;", $text); // normal bullet
			$text = str_replace("&#150;", "&ndash;", $text); // n-dash
			$text = str_replace("&#151;", "&mdash;", $text); // m-dash
			$text = str_replace("&#169;", "&copy;", $text); // copyright symbol
			$text = str_replace("&#223;", "&bull;", $text); // MS Word bullet
		} else {
			$text = str_replace("&#133;", "...", $text); // ellipses
			$text = str_replace("&#145;", "'", $text); // left single quote
			$text = str_replace("&#146;", "'", $text); // right single quote
			$text = str_replace("&#147;", "\"", $text); // left double quote
			$text = str_replace("&#148;", "\"", $text); // right double quote
			$text = str_replace("&#149;", " - ", $text); // normal bullet
			$text = str_replace("&#150;", " - ", $text); // n-dash
			$text = str_replace("&#151;", " -- ", $text); // m-dash
			$text = str_replace("&#169;", "(c)", $text); // copyright symbol
			$text = str_replace("&#223;", " - ", $text); // MS Word bullet
		}
		
		return $text;
		
	}
	
	
	function timestamp($language, $universal=0) {
		
		// get the user's time zone from the session
		$user_timezone = initialize("timezone", "session", "text");
		
		// get the server timezone from PHP, removing the effect of daylight savings time
		$server_timezone = (date("Z") / 60 / 60);
		if (date("I")) { $server_timezone--; }

		if ($universal) {
			// this returns the current time in Greenwich
			$timezone_correction = $server_timezone;
		} else {
			// this returns the current time in the user's location
			$timezone_correction = $server_timezone - $user_timezone;
		}
		
		if ($language == "PHP") {
			return date("Y-m-d H:i:s", time() - ($timezone_correction * 3600));
		} else if ($language == "SQL") {
			return "DATE_SUB(NOW(), INTERVAL $timezone_correction HOUR)";
		} else if ($language == "Unix") {
			return (time() - ($timezone_correction * 3600));
		}
		
	}
	
	
	function save_file($name, $content) {
		// if we're in demo mode, don't save anything
		if ($GLOBALS["demo"]) {
			return 1;
		}
		
		// $name is a relative path to the current working directory
		@$my_file = fopen($name, "w");
		if ($my_file) {
			fwrite($my_file, $content);
			fclose($my_file);
			$result = 1;
		} else {
			$result = 0;
		}
		return $result;
	}


	function rename_file($name_old, $name_new) {
		// rename a file, but avoid an error if it doesn't exist
		$success = 0;
		if ($name_old) {
			if (file_exists($name_old)) {
				$success = rename($name_old, $name_new);
			}
		}
		return $success;
	}


	function delete_file($file_location) {
		// delete a file, but avoid an error if it doesn't exist
		$success = 0;
		if ($file_location) {
			if (file_exists($file_location)) {
				$success = unlink($file_location);
			}
		}
		return $success;
	}


	function list_directory($directory, $types="directories", $recursive=0, $original_directory="") {
		
		// get a list of the directories and or files and turn the list into an array 
		// $directory - starting directory from which to search
		// $types - can be "directories," "files," or "all"
		// $recursive - determines whether subdirectories are searched
		// $original_directory - allows the function to return subdirectories but not the original directory as part of the file paths; leave this argument at the end and don't set it when calling the function
		
		// 2004-03-12 - initial development
		// 2006-10-02 - included subdirectories in file names, and searched directories recursively even when only files are returned
		
		if ($dir = opendir($directory)) {
			$file_list = array();
			while (($file = readdir($dir)) !== false) {
				// ignore the . and .. file references
				if ($file != "." && $file != "..") {
					// don't include the original directory in file names, but include any subdirectories
					if ($original_directory == "") { $original_directory = $directory; }
					$file_display = substr($directory."/".$file, (strlen($original_directory) + 1));
				
					// check whether this is a file or directory
					if (is_dir($directory."/".$file)) {
						// log the directory name if requested
						if (($types == "directories") || ($types == "all")) {
							$file_list[] = $file_display;
						}
						
						// if we're in a directory and we want recursive behavior, call this function again
						if ($recursive) {
							$file_list = array_merge($file_list, directory_list($directory."/".$file, $types, $recursive, $original_directory));
						}
					} else {
						// log the file name if requested
						if (($types == "files") || ($types == "all")) {
							$file_list[] = $file_display;
						}
					}
					
				}
			}
			closedir($dir);
			return $file_list;
		} else {
			return false;
		}
		
	}


	function get_position($string, $array) {
	
		// returns the index of an array element, something that ought to be built into PHP but isn't!
		// returns "" if not present
	
		$index = implode(",", array_keys($array, $string));
		return $index;
		
	}
	
	
	function redirect($page="", $secure=0, $window="window") {
		// makes a URL absolute if it's relative (for standards compliance), then loads it
		// if no arguments are provided, returns to the previous page
		if (($page == "")&&(isset($_SERVER["HTTP_REFERER"]))&&($_SERVER["HTTP_REFERER"] != "")) {
			$URL = $_SERVER["HTTP_REFERER"];
		} else if (strpos($page, "://") !== false) {
			$URL = $page;
		} else {
			$page = preg_replace("/^[\.\\/]+(.*)$/", "$1", $page); // remove a leading slash or ../ if it exists
			$URL = SITE_PROTOCOL."://".SITE_DOMAIN."/".$page;
		}
		if (headers_sent()) {
			// this is clunky, but it does the job if some output has already occurred
			print "<script language=\"JavaScript\"> $window.location = \"$URL\"; </script>";
		} else if (isset($_SERVER["HTTP_AJAX_REQUEST"])) {
			// if this is an AJAX request, pass the redirect value to the AJAX script know so it can redirect the entire page
			header("Ajax-Redirect: $URL");
		} else {
			// this is what we'll get most of the time
			header("Location: $URL");
		}
		exit;
	}


	function build_content_browser() {
	
		$output = array();
	
		// look for open directories and files saved in a cookie
		GLOBAL $open_groups, $open_paths;
		if (isset($_COOKIE["content_browser"])&&($_COOKIE["content_browser"])) {
			$cookie_string = urldecode($_COOKIE["content_browser"]);
			$cookie_array = explode_assoc("=", "&", $cookie_string);
			$open_groups = (isset($cookie_array["open_groups"])&&($cookie_array["open_groups"])) ? explode(",", $cookie_array["open_groups"]) : array() ;
			$open_paths = (isset($cookie_array["open_paths"])&&($cookie_array["open_paths"])) ? explode(",", $cookie_array["open_paths"]) : array() ;
		} else {
			$open_groups = array();
			$open_paths = array();
		}
		
		switch(CONTENT_TYPE) {
		
			case "contemplate":
				$version_check = check_contemplate_version();
				if ($version_check === 1) {
					$output[] = get_paths(CONTEMPLATE_CONTENT_DIRECTORIES);
				} else {
					$output[] = $version_check;
				}
				break;
				
			case "database":
				$output[] = get_paths(DATABASE_TABLES);
				break;
			
			case "dreamweaver":
				$output[] = get_paths(DREAMWEAVER_CONTENT_DIRECTORIES);
				break;
			
		}
		
		$output[] = "<a href=\"JavaScript:build_content_browser();\" class=\"link_refresh\">Refresh</a>";
		$output = implode("\n", $output);
		return $output;
	
	}
	
	
	function get_paths($group) {
		$output = array();

		GLOBAL $open_groups, $open_paths;
		if (!isset($open_groups)) { $open_groups = array(); }
		if (!isset($open_paths)) { $open_paths = array(); }
		if (strpos($group, ",") !== false) {
			// if multiple groups are specified, load each group
			$paths = explode(",", $group);
			$group = "";
		} else {
			// if just one group is specified, load all the paths in that group
			if (CONTENT_TYPE == "database") {
				$paths = array();
				db_connect_content();
				$query = "SELECT DISTINCT `".DATABASE_ID1_COLUMN."` FROM `".$group."`";
				$result = db_do_query($query);
				$count = db_num_rows($result);
				for ($i=0; $i<$count; $i++) {
					$row = db_get_row($result, $i);
					$paths[] = $row[DATABASE_ID1_COLUMN];
				}
				db_connect_editor();
			} else {
				$paths = list_directory(SITE_DIRECTORY."/$group", "all", 0);
			}
		}
		sort($paths);
		
		$content_file_extensions = explode(",", CONTENT_FILE_EXTENSIONS);
		foreach($paths as $path) {
			$file_extension = preg_replace("/^\w+\.(\w+?)$/", "$1", $path);
			if ( (in_array($file_extension, $content_file_extensions)) || (CONTENT_TYPE == "database") ) {
			
				$group_and_path = ($group) ? "$group/$path" : $path ;
				if ( ((CONTENT_TYPE == "database")&&(strpos($group_and_path, "/") === false)) || ((CONTENT_TYPE != "database")&&(is_dir(SITE_DIRECTORY."/$group_and_path"))) ) {
					$collapsable = (COLLAPSABLE_GROUPS);
					$expanded = (in_array("$group_and_path", $open_groups)||(!$collapsable));
					
					$locked_class = ($locked) ? "file_locked" : "" ;
					$collapsable_class = ($collapsable) ? "" : "not_collapsable" ;
					$expanded_class = ($expanded) ? "expanded" : "collapsed" ;

					if ($collapsable) {
						$output[] = "<a href=\"JavaScript:click_group('$group_and_path');\" id=\"group_$group_and_path\" class=\"content_browser_group $collapsable_class $expanded_class\">$path</a>";
					} else {
						$output[] = "<span class=\"content_browser_group $collapsable_class $expanded_class\">$path</span>";
					}
					$output[] = "<div id=\"paths_$group_and_path\" class=\"content_browser_paths $collapsable_class $expanded_class\">";
					if ($expanded) {
						$output[] = get_paths($group_and_path);
					}
					$output[] = "</div>";
				} else {
					if ((CONTENT_TYPE == "database")||(strpos($path, ".LCK") === false)) { // don't show Dreamweaver lock files
						$locked = (check_dreamweaver_lock($group_and_path));
						$collapsable = (COLLAPSABLE_GROUPS);
						$expanded = (in_array($group_and_path, $open_paths)||(!$collapsable));
						
						$locked_class = ($locked) ? "file_locked" : "" ;
						$collapsable_class = ($collapsable) ? "" : "not_collapsable" ;
						$expanded_class = ($expanded) ? "expanded" : "collapsed" ;
						
						if ($collapsable) {
							$output[] = "<a href=\"JavaScript:click_path('$group_and_path');\" id=\"path_$group_and_path\" class=\"content_browser_group $locked_class $collapsable_class $expanded_class\">$path</a>";
						} else {
							$output[] = "<span class=\"content_browser_group $locked_class $collapsable_class $expanded_class\">$path</span>";
						}
						$output[] = "<div id=\"paths_$group_and_path\" class=\"content_browser_paths $collapsable_class $expanded_class\">";
						if ($expanded) {
							$output[] = get_fields($group_and_path);
						}
						$output[] = "</div>";
					}
				}
				
			}
		}
		$output = implode("\n", $output);
		return $output;
	}
	
	
	function get_fields($path) {
		// first, get a list of the currently open (in progress, in review or scheduled) fields from our database
		$timezone = initialize("timezone", "session", "text");
		$fields_open = array();
		$query = "
			SELECT
				field,
				status.ID AS status_ID,
				status.name AS status,
				user.name AS editor,
				DATE_ADD(date, INTERVAL $timezone HOUR) AS last_saved,
				DATE_ADD(schedule, INTERVAL $timezone HOUR) AS schedule
			FROM content
			LEFT JOIN status ON status.ID=content.status
			LEFT JOIN user ON user.ID=content.user
			INNER JOIN (SELECT MAX(ID) AS ID FROM content GROUP BY path, field) AS content_latest ON content_latest.ID=content.ID
			WHERE path='$path'
			AND FIND_IN_SET(content.status, '1,2,3')
			ORDER BY last_saved
		";
		$result = db_do_query($query);
		$count = db_num_rows($result);
		for ($i=0; $i<$count; $i++) {
			$row = db_get_row($result, $i);
			$fields_open[$row["field"]] = $row;
		}
		
		if (CONTENT_TYPE == "contemplate") {

			// in order for the assembler functions to work, we have to declare all the globals that they use
			GLOBAL $error_log, $show_errors, $formulator_fill;
			GLOBAL $file_missing, $pre_render_files, $script_location;
			GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
			GLOBAL $check_URL;
			GLOBAL $field_marker;
			GLOBAL $script_location, $request, $URL_rewriting_flag;
	
			// now we can include the assembler script
			$_GET["skip_processing"] = 1;
			include_once CONTEMPLATE_DIRECTORY."/assembler.php";
	
			// this makes the assembler think we're running from the contemplate directory
			$script_location = CONTEMPLATE_DIRECTORY."/";
			
			// get the content field names and build a link for each one
			$output = array();
			$source = get_file("../".$path, "filesystem", 0);
			$file_type = (strpos($path, "xml") !== false) ? "xml" : "html" ;
			$fields = get_field_names($source, $file_type);
			//sort($fields);
		
		} else if (CONTENT_TYPE == "database") {
		
			list($table, $ID1) = explode("/", $path);
		
			$fields = array();
			db_connect_content();
			$query = "SELECT `".DATABASE_ID2_COLUMN."` FROM `".$table."` WHERE `".DATABASE_ID1_COLUMN."`='".$ID1."'";
			$result = db_do_query($query);
			$count = db_num_rows($result);
			for ($i=0; $i<$count; $i++) {
				$row = db_get_row($result, $i);
				$fields[] = $row[DATABASE_ID2_COLUMN];
			}
			db_connect_editor();
		
		} else if (CONTENT_TYPE == "dreamweaver") {
		
			$file = file_get_contents("../".$path);
			$match = preg_match_all("/<!--\s*InstanceBeginEditable\s*name=\"(\w+)\"\s*-->.*?<!--\s*InstanceEndEditable\s*-->/s", $file, $matches);
			if ($match) {
				$fields = $matches[1];
			} else {
				$fields = array();
			}
		
		}
		
		foreach($fields as $field) {
			if (in_array($field, array_keys($fields_open))) {
				// if this field is currently open, use a different style and add a hover box
				$hover = "";
				if ($fields_open[$field]["status_ID"] == 3) {
					$hover .= "Status: ".$fields_open[$field]["status"]." for ".$fields_open[$field]["schedule"]."\n";
				} else {
					$hover .= "Status: ".$fields_open[$field]["status"]."\n";
				}
				$hover .= "Editor: ".$fields_open[$field]["editor"]."\n";
				$hover .= "Last saved: ".$fields_open[$field]["last_saved"];
				$output[] = "<a href=\"JavaScript:click_field('$path', '$field');\" class=\"field_open\" title=\"$hover\">$field</a>";
				unset($fields_open[$field]);
			} else {
				$output[] = "<a href=\"JavaScript:click_field('$path', '$field');\">$field</a>";
			}
		}
		foreach($fields_open as $field_open) {
			// if there are any remaining open fields, they probably aren't in the content path yet, so show a link for those, too
			$hover = "";
			if ($field_open["status_ID"] == 3) {
				$hover .= "Status: ".$field_open["status"]." for ".$field_open["schedule"]."\n";
			} else {
				$hover .= "Status: ".$field_open["status"]."\n";
			}
			$hover .= "Editor: ".$field_open["editor"]."\n";
			$hover .= "Last saved: ".$field_open["last_saved"];
			$output[] = "<a href=\"JavaScript:click_field('$path', '".$field_open["field"]."');\" class=\"field_open\" title=\"$hover\">".$field_open["field"]."</a>";
		}
		if (CONTENT_TYPE != "dreamweaver") {
			// we don't allow adding new fields to Dreamweaver files because that would have to be done at the template level
			$output[] = "<a href=\"JavaScript:click_field('$path', '');\" class=\"link_new\">new field</a>";
		}
		$output = implode("<br />\n", $output);

		return $output;
	}
	
	
	function get_content($path, $field, $rollback=0, $last_published=0) {
		// since we're generating the status menus here, figure out which statuses are available to this user
		$role = initialize("role", "session", "integer");
		$available_statuses = ($role > 1) ? "1,2,3,4" : "1,2" ;
		
		if (CONTENT_TYPE == "contemplate") {
		
			// in order for the assembler functions to work, we have to declare all the globals that they use
			GLOBAL $error_log, $show_errors, $formulator_fill;
			GLOBAL $file_missing, $pre_render_files, $script_location;
			GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
			GLOBAL $field_marker, $file_missing, $content_file;
			GLOBAL $check_URL, $check_field_label;
			GLOBAL $script_location, $request, $URL_rewriting_flag;
	
			// now we can include the assembler script
			$_GET["skip_processing"] = 1;
			include_once CONTEMPLATE_DIRECTORY."/assembler.php";
	
			// this makes the assembler think we're running from the contemplate directory
			$script_location = CONTEMPLATE_DIRECTORY."/";
	
			// get the content field; we're going to need this to look for a background color, even if latest content is in the database
			$source = get_file("../".$path, "filesystem", 0);
			$file_type = (strpos($path, "xml") !== false) ? "xml" : "html" ;
			
		}
		
		// first, see if we're requesting a specific version of the content from the database
		if ($rollback) {
			$query = "SELECT content FROM content WHERE ID='$rollback'";
			$result = db_do_query($query);
			$count = db_num_rows($result);
			if ($count) {
				$row = db_get_row($result, 0);
				extract($row);
				$status_menu = make_menu("Status...", "status", "status", $available_statuses, 0, 0, 0, 0);
				$date_menu = make_date_menus("", "content_edit", "schedule", 0);
				$time_menu = make_time_menus("", "content_edit", "schedule", 15, 0);
				$ID = $rollback;
			}
		}
		
		// next, see if an unpublished version of this field is in our database; if so, return that content instead of the live content
		if ((!isset($content))&&(!$last_published)) {
			$timezone = initialize("timezone", "session", "text");
			$query = "SELECT content.ID, content, status, COALESCE(DATE_ADD(schedule, INTERVAL $timezone HOUR), '0000-00-00 00:00:00') AS schedule FROM content WHERE path='$path' AND field='$field' ORDER BY date DESC LIMIT 1";
			$result = db_do_query($query);
			$count = db_num_rows($result);
			if ($count) {
				$row = db_get_row($result, 0);
				if ($row["status"] < 4) {
					extract($row);
					$status_menu = make_menu("Status...", "status", "status", $available_statuses, 0, 0, 0, $status);
					list($schedule_date, $schedule_time) = explode(" ", $schedule);
					if ($schedule_date == "0000-00-00") { $schedule_time = ""; }
					$date_menu = make_date_menus("", "content_edit", "schedule", $schedule_date);
					$time_menu = make_time_menus("", "content_edit", "schedule", 15, $schedule_time);
				}
			}
		}
		
		// otherwise, get the content out of the site storage
		if (!isset($content)) {

			if (CONTENT_TYPE == "contemplate") {
			
				if ($field) {
					$content = get_field($source, $field, $file_type);
				} else {
					// if no field was specified, we're probably creating a new field
					$content = "";
				}
	
			} else if (CONTENT_TYPE == "database") {
			
				if ($field) {
					list($table, $ID1) = explode("/", $path);

					db_connect_content();
					$query = "SELECT `".DATABASE_CONTENT_COLUMN."` FROM `".$table."` WHERE `".DATABASE_ID1_COLUMN."`='".$ID1."' AND `".DATABASE_ID2_COLUMN."`='".$field."'";
					$result = db_do_query($query);
					$row = db_get_row($result, 0);
					$content = $row[DATABASE_CONTENT_COLUMN];
					db_connect_editor();
				} else {
					// if no field is found, we're probably creating a new field
					$content = "";
				}
			
			} else if (CONTENT_TYPE == "dreamweaver") {
			
				$file = file_get_contents("../".$path);
				$match = preg_match("/<!--\s*InstanceBeginEditable\s*name=\"$field\"\s*-->(.*?)<!--\s*InstanceEndEditable\s*-->/s", $file, $matches);
				if ($match) {
					$content = $matches[1];
				} else {
					$content = "";
				}
	
			}
		
			$status_menu = make_menu("Status...", "status", "status", $available_statuses, 0, 0, 0, 0);
			$date_menu = make_date_menus("", "content_edit", "schedule", 0);
			$time_menu = make_time_menus("", "content_edit", "schedule", 15, 0);
			$ID = "";
		}
		
		// look for a background color for this field
		if (CONTENT_TYPE == "contemplate") {
			$bg_color = get_field_bg_color($source, $field, $file_type);
		} else {
			$bg_color = "";
		}
		
		// remove slashes if not using magic quotes
		if (CONTENT_TYPE == "database") {
			if (!ini_get("magic_quotes_runtime")) {
				$content = stripslashes($content);
			}
		}

		// clean up high-ASCII characters, which will break the decodeURIComponent function in ajax_unserialize (and display badly in some browsers)
		$content = convert_characters($content);

		// set up the schedule menu for display
		$schedule_menu = "<div class=\"stack_left\">$date_menu</div><div class=\"stack_left\">$time_menu</div>";
		$schedule_menu = str_replace("&nbsp;", "", $schedule_menu);
		
		// get the name of the person using the file if applicable
		$lock_name = check_dreamweaver_lock($path);

		$response = compact("content", "bg_color", "status_menu", "schedule_menu", "lock_name", "ID");
		$response = ajax_serialize($response);
		return $response;
	}
	
	
	function save_content($path, $field, $content) {
		// clean up high-ASCII characters, which will break the decodeURIComponent function in ajax_unserialize (and display badly in some browsers)
		$content = convert_characters($content);

		if (CONTENT_TYPE == "contemplate") {

			// in order for the assembler functions to work, we have to declare all the globals that they use
			GLOBAL $error_log, $show_errors, $formulator_fill;
			GLOBAL $file_missing, $pre_render_files, $script_location;
			GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
			GLOBAL $field_marker, $file_missing, $content_file;
			GLOBAL $check_URL, $check_field_label;
			GLOBAL $script_location, $request, $URL_rewriting_flag;
	
			// now we can include the assembler script
			$_GET["skip_processing"] = 1;
			include_once CONTEMPLATE_DIRECTORY."/assembler.php";
	
			// this makes the assembler think we're running from the contemplate directory
			$script_location = CONTEMPLATE_DIRECTORY."/";
			
			// get the content file
			$source = get_file("../".$path, "filesystem", 0);
			$file_type = (strpos($path, "xml") !== false) ? "xml" : "html" ;
			
			// find out if this field is in the file yet
			$fields = get_field_names($source, $file_type);
			$action = (in_array($field, $fields)) ? "update" : "add" ;
	
			if ($action == "add") {
				// copy the last row of the file, add our new content, and append it to the file
				$last_field = $fields[(count($fields) - 1)];
				$last_row = get_row($source, $last_field, $file_type);
				$new_row = update_row($last_row, $file_type, $field, $content);
				$new_source = str_replace($last_row, $last_row."\n".$new_row, $source);
			} else if ($action == "update") {
				// update the existing row in place
				$original_row = get_row($source, $field, $file_type);
				$new_row = update_row($original_row, $file_type, $field, $content);
				$new_source = str_replace($original_row, $new_row, $source);
			}
	
			// save the new row to the file
			$result = save_file("../".$path, $new_source);
		
		} else if (CONTENT_TYPE == "database") {

			list($table, $ID1) = explode("/", $path);

			db_connect_content();

			// find out if this record is in the database yet
			$query = "SELECT `".DATABASE_CONTENT_COLUMN."` FROM `".$table."` WHERE `".DATABASE_ID1_COLUMN."`='".$ID1."' AND `".DATABASE_ID2_COLUMN."`='".$field."'";
			$result = db_do_query($query);
			$count = db_num_rows($result);
			$action = ($count) ? "update" : "add" ;
	
			if ($action == "add") {
				$query = "INSERT INTO `".$table."` (`".DATABASE_ID1_COLUMN."`, `".DATABASE_ID2_COLUMN."`, `".DATABASE_CONTENT_COLUMN."`) VALUES ('".$path."', '".$field."', '".$content."')";
			} else if ($action == "update") {
				$query = "UPDATE `".$table."` SET `".DATABASE_CONTENT_COLUMN."`='".$content."' WHERE `".DATABASE_ID1_COLUMN."`='".$ID1."' AND `".DATABASE_ID2_COLUMN."`='".$field."'";
			}
			$result = db_do_query($query);

			db_connect_editor();

		} else if (CONTENT_TYPE == "dreamweaver") {
		
			// update the content in the file
			$file = file_get_contents("../".$path);
			if (preg_match("/(.*<!--\s*InstanceBeginEditable\s*name=\"$field\"\s*-->)(.*?)(<!--\s*InstanceEndEditable\s*-->.*)/s", $file, $matches)) {
				// we use preg_match here because with preg_replace, any $dd variables in $content are interpreted as backreferences
				$new_file = $matches[1].$content.$matches[3];
			} else {
				$new_file = "";
			}

			// save the new content to the file
			if ($new_file) {
				$result = save_file("../".$path, $new_file);
			} else {
				$result = "";
			}

		}
		
		return $result;
	}
	
	
	function delete_content($path, $field, $content) {

		if (CONTENT_TYPE == "contemplate") {

			// in order for the assembler functions to work, we have to declare all the globals that they use
			GLOBAL $error_log, $show_errors, $formulator_fill;
			GLOBAL $file_missing, $pre_render_files, $script_location;
			GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
			GLOBAL $field_marker, $file_missing, $content_file;
			GLOBAL $check_URL, $check_field_label;
			GLOBAL $script_location, $request, $URL_rewriting_flag;
	
			// now we can include the assembler script
			$_GET["skip_processing"] = 1;
			include_once CONTEMPLATE_DIRECTORY."/assembler.php";
	
			// this makes the assembler think we're running from the contemplate directory
			$script_location = CONTEMPLATE_DIRECTORY."/";
			
			// get the content file
			$source = get_file("../".$path, "filesystem", 0);
			$file_type = (strpos($path, "xml") !== false) ? "xml" : "html" ;
			
			// remove the existing row from the file
			// this could just be a str_replace, but we want to get the trailing white space, too
			$original_row = get_row($source, $field, $file_type);
			$new_source = preg_replace("/(.*)".preg_quote($original_row, "/")."\s*[\n\r]*(.*)/s", "$1$2", $source);
	
			// save the new row to the file
			$result = save_file("../".$path, $new_source);
		
		} else if (CONTENT_TYPE == "database") {

			list($table, $ID1) = explode("/", $path);

			db_connect_content();
			$query = "DELETE FROM `".$table."` WHERE `".DATABASE_ID1_COLUMN."`='".$ID1."' AND `".DATABASE_ID2_COLUMN."`='".$field."'";
			$result = db_do_query($query);
			db_connect_editor();

		}

		return $result;

	}
	
	
	function check_contemplate_version() {
		// return 1 if the version is compatible, an error message if not
		
		$required_version = 1.3; // we assume that any sub-sub-version will be compatible
		$assembler_text = file_get_contents(CONTEMPLATE_DIRECTORY."/assembler.php");
		
		if (preg_match("/version (\d+\.\d+)/", $assembler_text, $matches)) {
			$installed_version = $matches[1];
		} else {
			$installed_version = 0;
		}

		if ($installed_version < $required_version) {
			return "<p class=\"feedback_negative\">Error: This version of ".EDITOR_NAME." requires version $required_version or newer of Contemplate.<br /><br />If you want to use a different content type, you can specify that in your config/constants.php file.</p>";
		} else {
			return 1;
		}
	}
	

	function check_dreamweaver_lock($path) {
		// if we see a Dreameaver lock file and none of the fields in this file are open by us, then the file is locked by Dreamweaver
		// return the name of the person who has it locked, or an empty string if it is not locked
		$lock_name = "";
		if ((USE_LCK_FILES)&&(CONTENT_TYPE != "database")) {
			if (file_exists(SITE_DIRECTORY."/".$path.".LCK")) {
				$query = "
					SELECT content.ID
					FROM content
					INNER JOIN (SELECT MAX(ID) AS ID FROM content GROUP BY path, field) AS content_latest ON content_latest.ID=content.ID
					WHERE path='$path'
					AND FIND_IN_SET(content.status, '1,2,3')
				";
				$result = db_do_query($query);
				$count = db_num_rows($result);
				if (!$count) {
					$lock_file = file_get_contents(SITE_DIRECTORY."/".$path.".LCK");
					list($lock_name, $lock_email) = explode("||", $lock_file);
				}
			}
		}
		return $lock_name;
	}
	
	
	function set_dreamweaver_lock($path) {
		// if we edit a file, create a Dreamweaver lock file to warn Dreamweaver, Contribute or other users not to edit it
		// return 1 if the file was created or already existed; return 0 if it should have been created but we were not able to create
		$result = 1;
		if ((USE_LCK_FILES)&&(CONTENT_TYPE != "database")) {
			if (!file_exists(SITE_DIRECTORY."/".$path.".LCK")) {
				$lock_name = initialize("username", "session", "text");
				$lock_email = get_value("user", "name", "email", $lock_name);
				$lock_file = "$lock_name||$lock_email";
				$file = fopen(SITE_DIRECTORY."/".$path.".LCK", "w");
				if ($file) {
					fwrite($file, $lock_file);
					fclose($file);
				} else {
					$result = 0;
				}
			}
		}
		return $result;
	}
	
	
	function clear_dreamweaver_lock($path) {
		// if there are no longer any unpublished fields in this file, remove the lock file now
		// return 1 if the file didn't exist or did not need to be deleted; return 0 if it should have been deleted but we were not able to delete
		$result = 1;
		if ((USE_LCK_FILES)&&(CONTENT_TYPE != "database")) {
			if (file_exists(SITE_DIRECTORY."/".$path.".LCK")) {
				$query = "
					SELECT content.ID
					FROM content
					INNER JOIN (SELECT MAX(ID) AS ID FROM content GROUP BY path, field) AS content_latest ON content_latest.ID=content.ID
					WHERE path='$path'
					AND FIND_IN_SET(content.status, '1,2,3')
				";
				$result = db_do_query($query);
				$count = db_num_rows($result);
				if (!$count) {
					$result = delete_file(SITE_DIRECTORY."/".$path.".LCK");
				}
			}
		}
		return $result;
	}
	
	
	function get_contemplate_field_names_UNUSED($file) {
		// in order for the assembler functions to work, we have to declare all the globals that they use
		GLOBAL $error_log, $show_errors, $formulator_fill;
		GLOBAL $file_missing, $pre_render_files, $script_location;
		GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
		GLOBAL $check_URL;
		GLOBAL $field_marker;
		GLOBAL $script_location, $request, $URL_rewriting_flag;

		// now we can include the assembler script
		$_GET["skip_processing"] = 1;
		include_once CONTEMPLATE_DIRECTORY."/assembler.php";

		// this makes the assembler think we're running from the contemplate directory
		$script_location = CONTEMPLATE_DIRECTORY."/";
		
		// get the content field names and build a link for each one
		$source = get_file("../".$file, "filesystem", 0);
		$file_type = (strpos($file, "xml") !== false) ? "xml" : "html" ;
		$fields = get_field_names($source, $file_type);

		return $fields;
	}
	
	
	function strip_template($file_contents) {
		// if we're running within the parent system in integration mode, strip out the header and footer
		if (THIS_PAGE_WRAPPED) {
			$file_contents = preg_replace("/(.*)<html.*\/title>(.*)<\/head>.*<\?php[^>]+include[^>]+HEADER_FILE[^>]+\?>(.*)<\?php[^>]+include[^>]+FOOTER_FILE[^>]+\?>.*/s", "$1$2$3", $file_contents);
		}
		return $file_contents;
	}
	
?>
